@@ -1,14 +1,19 @@
+use ec::hex_unless_dec;
+use ec::State;
+use regex::Regex;
 use std::cmp::min;
-use std::fmt;
-use std::num::NonZeroUsize;
 use std::fs::File;
 use std::io;
 use std::io::Read;
 use std::io::Write;
-use regex::Regex;
-use ansi_term::Color;
-use ansi_term::Color::Fixed;
-use unicode_segmentation::UnicodeSegmentation;
+use std::num::NonZeroUsize;
+
+// TODO This is deprecated and should be
+// replaced with
+//     ec = {package = "edhex_core", version = "0.1.0}
+// in Cargo.toml.  But that's only going to
+// work for after Rust 1.26.0  Far enough in the future, use the Cargo.toml way.
+extern crate edhex_core as ec;
 
 
 macro_rules! skip_bad_range {
@@ -21,67 +26,6 @@ macro_rules! skip_bad_range {
 }
 
 
-/* Byte formatting stuff lifted from hexyl */
-const COLOR_NULL: Color = Fixed(1);
-const COLOR_ASCII_PRINTABLE: Color = Color::Cyan;
-const COLOR_ASCII_WHITESPACE: Color = Color::Green;
-const COLOR_ASCII_OTHER: Color = Color::Purple;
-const COLOR_NONASCII: Color = Color::Yellow;
-
-pub enum ByteCategory {
-    Null,
-    AsciiPrintable,
-    AsciiWhitespace,
-    AsciiOther,
-    NonAscii,
-}
-
-#[derive(Copy, Clone)]
-struct Byte(u8);
-
-impl Byte {
-    fn category(self) -> ByteCategory {
-        if self.0 == 0x00 {
-            ByteCategory::Null
-        } else if self.0.is_ascii_graphic() {
-            ByteCategory::AsciiPrintable
-        } else if self.0.is_ascii_whitespace() {
-            ByteCategory::AsciiWhitespace
-        } else if self.0.is_ascii() {
-            ByteCategory::AsciiOther
-        } else {
-            ByteCategory::NonAscii
-        }
-    }
-
-    fn color(self) -> &'static Color {
-        use crate::ByteCategory::*;
-
-        match self.category() {
-            Null => &COLOR_NULL,
-            AsciiPrintable => &COLOR_ASCII_PRINTABLE,
-            AsciiWhitespace => &COLOR_ASCII_WHITESPACE,
-            AsciiOther => &COLOR_ASCII_OTHER,
-            NonAscii => &COLOR_NONASCII,
-        }
-    }
-
-    fn as_char(self) -> char {
-        use crate::ByteCategory::*;
-
-        match self.category() {
-
-            /* hexyl uses 0 here, depending on color to distinguish */
-            Null => '•',
-            AsciiPrintable => self.0 as char,
-            AsciiWhitespace if self.0 == 0x20 => ' ',
-            AsciiWhitespace => '_',
-            AsciiOther => '•',
-            NonAscii => '×',
-        }
-    }
-}
-
 #[derive(Debug)]
 struct Command {
     range: (usize, usize),
@@ -92,7 +36,7 @@ struct Command {
 
 fn print_help() {
     print!("Input/output is hex unless toggled to decimal with 'x'
-?            This help
+h            This help
 <Enter>      Print current byte(s) and move forward to next set of byte(s)
 3d4          Move to byte number 3d4 and print from there
 +            Move 1 byte forward and print from there
@@ -103,6 +47,8 @@ fn print_help() {
 $            Move to last byte and print it
 /deadbeef    If the bytes de ad be ef exist after the current index, move there
                and print.
+?deadbeef    If the bytes de ad be ef exist before the current index, move there
+               and print.
 k            Delete (kill) byte at current index and print new line of byte(s)
 7dk          Move to byte 7d, delete that byte, and print from there.
 1d,72k       Move to byte 1d, delete bytes 1d - 72 inclusive, and print from there.
@@ -128,56 +74,6 @@ q            quit
 }
 
 
-fn string_from_bytes(bytes:&[u8]) -> String {
-    let mut to_return = "".to_owned();
-    for &byte in bytes {
-        to_return += &padded_byte(byte);
-    }
-    return to_return;
-}
-
-
-fn index_of_bytes(needle:&[u8], haystack:&[u8]) -> Option<usize> {
-    let needle_num_bytes = if needle.len() == 0 {
-        return None;
-    }
-    else {
-        needle.len()
-    };
-
-    for index in 0..haystack.len() {
-        let range = (index, index + needle_num_bytes - 1);
-
-        if bad_range(&haystack.to_vec(), range) {
-            return None;
-        }
-
-
-        if &haystack[range.0..=range.1] == needle {
-            return Some(index);
-        }
-        else {
-
-        }
-    }
-
-    None
-}
-
-
-fn bytes_from_string(nibbles_s:&str) -> Result<Vec<u8>, String> {
-    // TODO Allow general whitespace, not just literal spaces
-    let re_bytes = Regex::new(r"^ *([0-9a-fA-F][0-9a-fA-F] *)* *$").unwrap();
-    if re_bytes.is_match(&nibbles_s) {
-        let nibbles_v:Vec<String> = nibbles_s.replace(" ", "").chars().map(|x| x.to_string()).collect();
-        Ok(nibbles_v.chunks(2).map(|x| x.join("")).map(|x| u8::from_str_radix(&x, 16).unwrap()).collect())
-    }
-    else {
-        Err(format!("Couldn't interpret '{}' as a sequence of bytes", &nibbles_s))
-    }
-}
-
-
 fn read_bytes_from_user() -> Result<Vec<u8>, String> {
     print!("> ");
     io::stdout().flush().unwrap();
@@ -188,43 +84,15 @@ fn read_bytes_from_user() -> Result<Vec<u8>, String> {
         }
     };
 
-    bytes_from_string(&input)
+    ec::bytes_from_string(&input)
 }
 
 
-// TODO: This should take a usize..=usize range object
-fn bad_range(bytes: &Vec<u8>, range: (usize, usize)) -> bool {
-    bytes.len() == 0 || range.1 >= bytes.len()
-}
-
-
-/// Returns new index
-fn move_to(state:&mut State, index:usize) -> Result<usize, String> {
-    if state.empty() {
-        Err("Empty file".to_owned())
-    }
-    else {
-        let _max_index = match state.max_index() {
-            Ok(max) => max,
-            Err(error) => {
-                return Err(error);
-            },
-        };
-
-        if index > _max_index {
-            Err(format!("{} > {} = maximum index", hex_unless_dec_with_radix(index, state.radix), hex_unless_dec_with_radix(_max_index, state.radix)))
-        }
-        else {
-            state.index = index;
-            Ok(index)
-        }
-    }
-}
 
 
 impl Command {
     fn bad_range(&self, all_bytes: &Vec<u8>) -> bool {
-        bad_range(all_bytes, self.range)
+        ec::bad_range(all_bytes, self.range)
     }
 
 
@@ -235,10 +103,10 @@ impl Command {
         let re_blank_line = Regex::new(r"^ *$").unwrap();
         let re_pluses = Regex::new(r"^ *(?P<pluses>\++) *$").unwrap();
         let re_minuses = Regex::new(r"^ *(?P<minuses>\-+) *$").unwrap();
-        let re_search = Regex::new(r"^ */(?P<bytes>[0-9a-fA-F]+) *$").unwrap();
+        let re_search = Regex::new(r"^ *(?P<direction>[/?]) *(?P<bytes>[0-9a-fA-F]+) *$").unwrap();
         let re_search_kill = Regex::new(r"^ */(?P<bytes>[0-9a-fA-F]+)/k *$").unwrap();
         let re_search_insert = Regex::new(r"^ */(?P<bytes>[0-9a-fA-F]+)/i *$").unwrap();
-        let re_single_char_command = Regex::new(r"^ *(?P<command>[?mnopsxqwik]).*$").unwrap();
+        let re_single_char_command = Regex::new(r"^ *(?P<command>[hmnopsxqwik]).*$").unwrap();
         let re_range = Regex::new(r"^ *(?P<begin>[0-9a-fA-F.$]+) *, *(?P<end>[0-9a-fA-F.$]+) *(?P<the_rest>.*) *$").unwrap();
         let re_specified_index = Regex::new(r"^ *(?P<index>[0-9A-Fa-f.$]+) *(?P<the_rest>.*) *$").unwrap();
         let re_offset_index = Regex::new(r"^ *(?P<sign>[-+])(?P<offset>[0-9A-Fa-f]+) *(?P<the_rest>.*) *$").unwrap();
@@ -297,7 +165,7 @@ impl Command {
         let caps = re.captures(line);
 
         if is_pluses {
-            let num_pluses = num_graphemes(caps.unwrap().name("pluses").unwrap().as_str());
+            let num_pluses = ec::num_graphemes(caps.unwrap().name("pluses").unwrap().as_str());
             Ok(Command{
                 range: (num_pluses, num_pluses),
                 command: 'G',
@@ -306,9 +174,9 @@ impl Command {
         }
 
         else if is_search_insert {
-            match bytes_from_string(caps.unwrap().name("bytes").unwrap().as_str()) {
+            match ec::bytes_from_string(caps.unwrap().name("bytes").unwrap().as_str()) {
                 Ok(needle) => {
-                    if let Some(offset) = index_of_bytes(&needle, &state.all_bytes[state.index..]) {
+                    if let Some(offset) = ec::index_of_bytes(&needle, &state.all_bytes[state.index..], true) {
                         Ok(Command{
                             range: (state.index + offset, state.index + offset),
                             command: 'i',
@@ -316,7 +184,7 @@ impl Command {
                         })
                     }
                     else {
-                        Err(format!("{} not found", string_from_bytes(&needle)))
+                        Err(format!("{} not found", ec::string_from_bytes(&needle)))
                     }
                 },
                 Err(error) => {
@@ -326,7 +194,7 @@ impl Command {
         }
 
         else if is_search_kill {
-            match bytes_from_string(caps.unwrap().name("bytes").unwrap().as_str()) {
+            match ec::bytes_from_string(caps.unwrap().name("bytes").unwrap().as_str()) {
                 Ok(needle) => {
                     let needle_num_bytes = if needle.len() == 0 {
                         return Err("Searching for empty string".to_owned());
@@ -335,7 +203,7 @@ impl Command {
                         needle.len()
                     };
 
-                    if let Some(offset) = index_of_bytes(&needle, &state.all_bytes[state.index..]) {
+                    if let Some(offset) = ec::index_of_bytes(&needle, &state.all_bytes[state.index..], true) {
                         Ok(Command{
                             range: (state.index + offset, state.index + offset + needle_num_bytes - 1),
                             command: 'k',
@@ -343,7 +211,7 @@ impl Command {
                         })
                     }
                     else {
-                        Err(format!("{} not found", string_from_bytes(&needle)))
+                        Err(format!("{} not found", ec::string_from_bytes(&needle)))
                     }
                 },
                 Err(error) => {
@@ -353,17 +221,34 @@ impl Command {
         }
 
         else if is_search {
-            match bytes_from_string(caps.unwrap().name("bytes").unwrap().as_str()) {
+            let caps = caps.unwrap();
+            let forward = caps.name("direction").unwrap().as_str() == "/";
+            match ec::bytes_from_string(caps.name("bytes").unwrap().as_str()) {
                 Ok(needle) => {
-                    if let Some(offset) = index_of_bytes(&needle, &state.all_bytes[state.index..]) {
-                        Ok(Command{
-                            range: (state.index + offset, state.index + offset),
-                            command: 'g',
-                            args: vec![],
-                        })
+                    let haystack = if forward {
+                        &state.all_bytes[state.index..]
                     }
                     else {
-                        Err(format!("{} not found", string_from_bytes(&needle)))
+                        &state.all_bytes[..state.index]
+                    };
+                    if let Some(offset) = ec::index_of_bytes(&needle, haystack, forward) {
+                        if forward {
+                            Ok(Command{
+                                range: (state.index + offset, state.index + offset),
+                                command: 'g',
+                                args: vec![],
+                            })
+                        }
+                        else {
+                            Ok(Command{
+                                range: (offset, offset),
+                                command: 'g',
+                                args: vec![],
+                            })
+                        }
+                    }
+                    else {
+                        Err(format!("{} not found", ec::string_from_bytes(&needle)))
                     }
                 },
                 Err(error) => {
@@ -373,7 +258,7 @@ impl Command {
         }
 
         else if is_minuses {
-            let num_minuses = num_graphemes(caps.unwrap().name("minuses").unwrap().as_str());
+            let num_minuses = ec::num_graphemes(caps.unwrap().name("minuses").unwrap().as_str());
             Ok(Command{
                 range: (num_minuses, num_minuses),
                 command: 'H',
@@ -556,16 +441,6 @@ impl Command {
 }
 
 
-fn string_from_radix(radix: u32) -> String {
-    if radix == 10 {
-        "decimal".to_owned()
-    }
-    else {
-        "hex".to_owned()
-    }
-}
-
-
 fn get_input_or_die() -> Result<String, i32> {
     let mut input = String::new();
     match io::stdin().read_line(&mut input) {
@@ -587,326 +462,6 @@ fn get_input_or_die() -> Result<String, i32> {
 }
 
 
-fn num_bytes_or_die(open_file: &Option<std::fs::File>) -> Result<usize, i32> {
-    if open_file.is_none() {
-        return Ok(0);
-    }
-
-    let metadata = open_file.as_ref().unwrap().metadata();
-    match metadata {
-        Ok(metadata) => {
-            Ok(metadata.len() as usize)
-        }
-        Err(_) => {
-            println!("Couldn't find file size");
-            Err(2)
-        }
-    }
-}
-
-
-fn formatted_byte(byte:u8, color:bool) -> String {
-    if color {
-        Byte(byte).color().paint(padded_byte(byte)).to_string()
-    }
-    else {
-        padded_byte(byte)
-    }
-}
-
-
-fn chared_byte(byte:u8, color:bool) -> String {
-    if color {
-        Byte(byte).color().paint(String::from(Byte(byte).as_char())).to_string()
-    }
-    else {
-        Byte(byte).as_char().to_string()
-    }
-}
-
-
-fn padded_byte(byte:u8) -> String {
-    format!("{:02x}", byte)
-}
-
-
-fn max_bytes_line(bytes:&[u8], width:NonZeroUsize) -> usize {
-    if bytes.len() == 0 {
-        0
-    }
-    else {
-        (bytes.len() - 1) / usize::from(width)
-    }
-}
-
-
-fn bytes_line_range(bytes:&[u8], line_number:usize, width:NonZeroUsize) -> std::ops::Range<usize> {
-    let width = usize::from(width);
-    if line_number * width < bytes.len() {
-        let end_index = min(bytes.len(), line_number * width + width);
-        line_number * width..end_index
-    }
-    else {
-        0..0
-    }
-}
-
-
-fn bytes_line(bytes:&[u8], line_number:usize, width:NonZeroUsize) -> &[u8] {
-    &bytes[bytes_line_range(bytes, line_number, width)]
-}
-
-
-fn chars_line(bytes:&[u8], line_number:usize, width:NonZeroUsize, color:bool) -> String {
-    let mut to_return:String = "".to_owned();
-    for index in bytes_line_range(bytes, line_number, width) {
-        to_return += &String::from(chared_byte(bytes[index], color));
-    }
-    to_return
-}
-
-
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-
-    #[test]
-    fn test_padded_byte() {
-        assert_eq!(padded_byte(2), "02");
-        assert_eq!(padded_byte(10), "0a");
-    }
-
-    #[test]
-    fn test_index_of_bytes() {
-        let haystack = vec![0xde, 0xad, 0xbe, 0xef];
-        assert_eq!(index_of_bytes(&vec![], &haystack), None);
-        assert_eq!(index_of_bytes(&vec![0xad, 0xbe], &haystack), Some(1));
-        assert_eq!(index_of_bytes(&vec![0xad, 0xbe, 0xef], &haystack), Some(1));
-        assert_eq!(index_of_bytes(&vec![0xad, 0xbe, 0xef, 0xef], &haystack), None);
-        assert_eq!(index_of_bytes(&vec![0xde,], &haystack), Some(0));
-    }
-
-
-    #[test]
-    fn test_max_bytes_line() {
-        let _1 = NonZeroUsize::new(1).unwrap();
-        let _2 = NonZeroUsize::new(2).unwrap();
-        let _3 = NonZeroUsize::new(3).unwrap();
-        let _4 = NonZeroUsize::new(4).unwrap();
-        let _5 = NonZeroUsize::new(5).unwrap();
-        let _6 = NonZeroUsize::new(6).unwrap();
-        let _7 = NonZeroUsize::new(7).unwrap();
-        let _8 = NonZeroUsize::new(8).unwrap();
-        let bytes = vec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13];
-        assert_eq!(max_bytes_line(&bytes, _1), 12);
-        let bytes = vec![8, 6, 7, 5, 3, 0, 9,];
-        assert_eq!(max_bytes_line(&bytes, _1), 6);
-        assert_eq!(max_bytes_line(&bytes, _2), 3);
-        assert_eq!(max_bytes_line(&bytes, _3), 2);
-        assert_eq!(max_bytes_line(&bytes, _4), 1);
-        assert_eq!(max_bytes_line(&bytes, _5), 1);
-        assert_eq!(max_bytes_line(&bytes, _6), 1);
-        assert_eq!(max_bytes_line(&bytes, _7), 0);
-        assert_eq!(max_bytes_line(&bytes, _8), 0);
-        let bytes = vec![8, 6, 7, 5, 3, 0,];
-        assert_eq!(max_bytes_line(&bytes, _1), 5);
-        assert_eq!(max_bytes_line(&bytes, _2), 2);
-        assert_eq!(max_bytes_line(&bytes, _3), 1);
-        assert_eq!(max_bytes_line(&bytes, _4), 1);
-        assert_eq!(max_bytes_line(&bytes, _5), 1);
-        assert_eq!(max_bytes_line(&bytes, _6), 0);
-        assert_eq!(max_bytes_line(&bytes, _7), 0);
-        assert_eq!(max_bytes_line(&bytes, _8), 0);
-        let bytes = vec![8, 6, 7,];
-        assert_eq!(max_bytes_line(&bytes, _1), 2);
-        assert_eq!(max_bytes_line(&bytes, _2), 1);
-        assert_eq!(max_bytes_line(&bytes, _3), 0);
-        assert_eq!(max_bytes_line(&bytes, _4), 0);
-        assert_eq!(max_bytes_line(&bytes, _5), 0);
-        let bytes = vec![8, 6,];
-        assert_eq!(max_bytes_line(&bytes, _1), 1);
-        assert_eq!(max_bytes_line(&bytes, _2), 0);
-        assert_eq!(max_bytes_line(&bytes, _3), 0);
-        assert_eq!(max_bytes_line(&bytes, _4), 0);
-        let bytes = vec![8,];
-        assert_eq!(max_bytes_line(&bytes, _1), 0);
-        assert_eq!(max_bytes_line(&bytes, _2), 0);
-        assert_eq!(max_bytes_line(&bytes, _3), 0);
-        assert_eq!(max_bytes_line(&bytes, _4), 0);
-        let bytes = vec![];
-        assert_eq!(max_bytes_line(&bytes, _1), 0);
-        assert_eq!(max_bytes_line(&bytes, _2), 0);
-        assert_eq!(max_bytes_line(&bytes, _3), 0);
-        assert_eq!(max_bytes_line(&bytes, _4), 0);
-    }
-
-    #[test]
-    fn test_num_graphemes() {
-        assert_eq!(num_graphemes("hey, there"), 10);
-        assert_eq!(num_graphemes("दीपक"), 3);
-        assert_eq!(num_graphemes("ﷺ"), 1);
-        assert_eq!(num_graphemes("père"), 4);
-    }
-
-    #[test]
-    fn test_bytes_line() {
-        let bytes = vec![];
-        let _1 = NonZeroUsize::new(1).unwrap();
-        let _2 = NonZeroUsize::new(2).unwrap();
-        let _3 = NonZeroUsize::new(3).unwrap();
-        let _4 = NonZeroUsize::new(4).unwrap();
-        let _5 = NonZeroUsize::new(5).unwrap();
-        let _6 = NonZeroUsize::new(6).unwrap();
-        let _7 = NonZeroUsize::new(7).unwrap();
-        let _8 = NonZeroUsize::new(8).unwrap();
-        assert_eq!(bytes_line(&bytes, 0, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _2).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 1, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 1, _2).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 2, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 2, _2).to_owned(), vec![]);
-        let bytes = vec![8, 6, 7, 5, 3, 0, 9,];
-        assert_eq!(bytes_line(&bytes, 0, _1).to_owned(), vec![8,]);
-        assert_eq!(bytes_line(&bytes, 1, _1).to_owned(), vec![6,]);
-        assert_eq!(bytes_line(&bytes, 2, _1).to_owned(), vec![7,]);
-        assert_eq!(bytes_line(&bytes, 3, _1).to_owned(), vec![5,]);
-        assert_eq!(bytes_line(&bytes, 4, _1).to_owned(), vec![3,]);
-        assert_eq!(bytes_line(&bytes, 5, _1).to_owned(), vec![0,]);
-        assert_eq!(bytes_line(&bytes, 6, _1).to_owned(), vec![9,]);
-        assert_eq!(bytes_line(&bytes, 7, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 8, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 9, _1).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _2).to_owned(), vec![8, 6,]);
-        assert_eq!(bytes_line(&bytes, 1, _2).to_owned(), vec![7, 5,]);
-        assert_eq!(bytes_line(&bytes, 2, _2).to_owned(), vec![3, 0,]);
-        assert_eq!(bytes_line(&bytes, 3, _2).to_owned(), vec![9,]);
-        assert_eq!(bytes_line(&bytes, 4, _2).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 5, _2).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _3).to_owned(), vec![8, 6, 7,]);
-        assert_eq!(bytes_line(&bytes, 1, _3).to_owned(), vec![5, 3, 0,]);
-        assert_eq!(bytes_line(&bytes, 2, _3).to_owned(), vec![9,]);
-        assert_eq!(bytes_line(&bytes, 3, _3).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 4, _3).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _4).to_owned(), vec![8, 6, 7, 5,]);
-        assert_eq!(bytes_line(&bytes, 1, _4).to_owned(), vec![3, 0, 9,]);
-        assert_eq!(bytes_line(&bytes, 2, _4).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 3, _4).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 4, _4).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _5).to_owned(), vec![8, 6, 7, 5, 3,]);
-        assert_eq!(bytes_line(&bytes, 1, _5).to_owned(), vec![0, 9,]);
-        assert_eq!(bytes_line(&bytes, 2, _5).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 3, _5).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _6).to_owned(), vec![8, 6, 7, 5, 3, 0,]);
-        assert_eq!(bytes_line(&bytes, 1, _6).to_owned(), vec![9,]);
-        assert_eq!(bytes_line(&bytes, 2, _6).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 3, _6).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _7).to_owned(), vec![8, 6, 7, 5, 3, 0, 9,]);
-        assert_eq!(bytes_line(&bytes, 1, _7).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 2, _7).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 3, _7).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 0, _8).to_owned(), vec![8, 6, 7, 5, 3, 0, 9,]);
-        assert_eq!(bytes_line(&bytes, 1, _8).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 2, _8).to_owned(), vec![]);
-        assert_eq!(bytes_line(&bytes, 3, _8).to_owned(), vec![]);
-    }
-}
-
-
-fn print_state(state:&State) {
-    println!("At byte {} of {}", lino(&state),
-            hex_unless_dec_with_radix(state.all_bytes.len(), state.radix));
-    if state.show_byte_numbers {
-        println!("Printing byte numbers in {}",
-            string_from_radix(state.radix));
-    };
-    if state.show_chars {
-        println!("Printing char representations after bytes");
-    };
-    println!("Interpreting input numbers as {}",
-            string_from_radix(state.radix));
-    println!("Printing a newline every {} bytes",
-            hex_unless_dec_with_radix(usize::from(state.width), state.radix));
-    if state.unsaved_changes {
-        println!("Unwritten changes");
-    }
-    else {
-        println!("No unwritten changes");
-    }
-}
-
-
-/// returns index of the byte in the 0-th column of the last row printed
-fn print_bytes(state:&State, range:(usize, usize)) -> Option<usize> {
-    if state.empty() {
-        return None;
-    }
-
-    let max = state.max_index();
-    if max.is_err() {
-        println!("? ({:?})", max);
-        return None;
-    }
-    let max = max.unwrap();
-
-    let from = range.0;
-    let to = min(max, range.1);
-    if bad_range(&state.all_bytes, (from, to)) {
-      println!("? (Bad range: ({}, {}))", range.0, range.1);
-      return None;
-    }
-
-    let bytes = &state.all_bytes[from..=to];
-    let max_bytes_line_num = max_bytes_line(bytes, state.width);
-    let mut left_col_byte_num = from;
-    for bytes_line_num in 0..=max_bytes_line_num {
-        if bytes_line_num != 0 {
-            left_col_byte_num += usize::from(state.width);
-        }
-        if state.show_byte_numbers {
-            if state.radix == 10 {
-                print!("{:>5}{}|", left_col_byte_num, state.n_padding);
-            }
-            else {
-                print!("{:>5x}{}|", left_col_byte_num, state.n_padding);
-            }
-        }
-        let cur_line = bytes_line(bytes, bytes_line_num, state.width);
-        let with_color = cur_line.iter().map(|x| formatted_byte(*x, true))
-                .collect::<Vec<String>>().join(" ");
-        let sans_color = cur_line.iter().map(|x| formatted_byte(*x, false))
-                .collect::<Vec<String>>().join(" ");
-        if state.color {
-            print!("{}", with_color);
-        }
-        else {
-            print!("{}", sans_color);
-        }
-        let expected_length = usize::from(state.width) * 3 - 1;
-        for _ in num_graphemes(&sans_color)..expected_length {
-            print!(" ");
-        }
-        // TODO Do this padding stuff format!  Unclear why previous attempts
-        // have failed.
-        if state.show_chars {
-            print!("|   {}", chars_line(bytes, bytes_line_num, state.width,
-                state.color));
-        }
-        println!();
-        left_col_byte_num = from + bytes_line_num * usize::from(state.width);
-    }
-    Some(left_col_byte_num)
-}
-
-
-/// .len gives the number of bytes
-/// .chars.count() gives the number of characters (which counts è as two characters.
-/// The human concept is unicode "graphemes" or "glyphs" defined to be what
-/// think they are.
-fn num_graphemes(unicode_string: &str) -> usize {
-    return unicode_string.graphemes(true).count();
-}
-
 fn number_dot_dollar(index:usize, _max_index:usize, input:&str, radix:u32)
         -> Result<usize, String> {
     match input {
@@ -924,85 +479,6 @@ fn number_dot_dollar(index:usize, _max_index:usize, input:&str, radix:u32)
 }
 
 
-fn hex_unless_dec_with_radix(number:usize, radix:u32) -> String {
-    let letter = if radix == 10 {
-        'd'
-    }
-    else {
-        'x'
-    };
-
-    format!("0{}{}", letter, hex_unless_dec(number, radix))
-}
-
-
-fn hex_unless_dec(number:usize, radix:u32) -> String {
-    if radix == 10 {
-        format!("{}", number)
-    }
-    else {
-        format!("{:x}", number)
-    }
-}
-
-
-fn lino(state:&State) -> String {
-    hex_unless_dec_with_radix(state.index, state.radix)
-}
-
-
-struct State {
-    radix: u32,
-    show_byte_numbers: bool,
-    show_chars: bool,
-    unsaved_changes: bool,
-    filename: String,
-
-    show_prompt: bool,
-    color: bool,
-
-    /* Current byte number, 0 to len -1 */
-    index: usize,
-
-    width: NonZeroUsize,
-
-    /* The bytes in memory */
-    all_bytes: Vec<u8>,
-
-    /* Spaces to put between a byte number and a byte when displaying */
-    n_padding: String,
-}
-
-
-impl fmt::Debug for State {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "radix: {}|unsaved_changes: {}|show_byte_numbers: {}|show_chars: {}|index: {}|width: {}|n_padding: '{}'|filename: {}|",
-                self.radix, self.unsaved_changes, self.show_byte_numbers,
-                self.show_chars, self.index, self.width, self.n_padding,
-                self.filename)
-    }
-}
-
-
-impl State {
-    fn empty(&self) -> bool {
-        self.all_bytes.len() == 0
-    }
-
-    fn range(&self) -> (usize, usize) {
-        (self.index, self.index + usize::from(self.width) - 1)
-    }
-
-    fn max_index(&self) -> Result<usize, String> {
-        if self.all_bytes.len() == 0 {
-            Err("No bytes, so no max index.".to_owned())
-        }
-        else {
-            Ok(self.all_bytes.len() - 1)
-        }
-    }
-}
-
 /// Returns new index number
 fn minuses(state:&mut State, num_minuses:usize) -> Result<usize, String> {
     if state.empty() {
@@ -1016,7 +492,7 @@ fn minuses(state:&mut State, num_minuses:usize) -> Result<usize, String> {
     }
     else {
         state.index -= num_minuses;
-        print_bytes(&state, state.range());
+        state.print_bytes(state.range());
         Ok(state.index)
     }
 }
@@ -1037,7 +513,7 @@ fn pluses(state:&mut State, num_pluses:usize) -> Result<usize, String> {
                 }
                 else {
                     state.index += num_pluses;
-                    print_bytes(&state, state.range());
+                    state.print_bytes(state.range());
                     Ok(state.index)
                 }
             },
@@ -1064,7 +540,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
         }
     };
 
-    let original_num_bytes = match num_bytes_or_die(&file) {
+    let original_num_bytes = match ec::num_bytes_or_die(&file) {
         Ok(num_bytes) => {
             num_bytes
         },
@@ -1094,7 +570,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
 
 
     // TODO Below here should be a function called main_loop()
-    let mut state = State{
+    let mut state = ec::State{
         radix: 16,
         filename: filename.to_owned(),
         show_byte_numbers: true,
@@ -1110,11 +586,10 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
     };
 
     if !quiet {
-        println!("{} bytes\n? for help\n",
-                hex_unless_dec(state.all_bytes.len(), state.radix));
-        print_state(&state);
+        println!("h for help\n");
+        println!("{}", state);
         println!();
-        print_bytes(&state, state.range());
+        state.print_bytes(state.range());
     }
 
     loop {
@@ -1142,9 +617,9 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
 
                     /* Go to */
                     'g' => {
-                        match move_to(&mut state, command.range.0) {
+                        match ec::move_to(&mut state, command.range.0) {
                             Ok(_) => {
-                                print_bytes(&state, state.range());
+                                state.print_bytes(state.range());
                             },
                             Err(error) => {
                                 println!("? ({})", error);
@@ -1197,7 +672,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
                                 }
                                 state.all_bytes = new;
                                 state.unsaved_changes = true;
-                                print_bytes(&state, state.range());
+                                state.print_bytes(state.range());
                             },
                             Err(error) => {
                                 println!("? ({})", error);
@@ -1206,7 +681,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
                     },
 
                     /* Help */
-                    '?'|'h' => {
+                    'h' => {
                         print_help();
                     },
 
@@ -1222,7 +697,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
                         right_half = right_half.split_off(command.range.1 - command.range.0 + 1);
                         state.all_bytes.append(&mut right_half);
                         state.index = command.range.0;
-                        print_bytes(&state, state.range());
+                        state.print_bytes(state.range());
                     },
 
                     /* Toggle showing char representations of bytes */
@@ -1281,7 +756,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
                                 }
                                 else {
                                     state.index = first_byte_to_show_index;
-                                    print_bytes(&state, (first_byte_to_show_index,
+                                    state.print_bytes((first_byte_to_show_index,
                                             last_byte_to_show_index));
                                 }
                             },
@@ -1300,7 +775,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
 
                         skip_bad_range!(command, state.all_bytes);
                         state.index = command.range.0;
-                        if let Some(last_left_col_index) = print_bytes(&state, state.range()) {
+                        if let Some(last_left_col_index) = state.print_bytes(state.range()) {
                             state.index = last_left_col_index;
                         }
                         else {
@@ -1317,7 +792,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
 
                         skip_bad_range!(command, state.all_bytes);
                         state.index = command.range.0;
-                        if let Some(last_left_col_index) = print_bytes(&state, (command.range.0, command.range.1)) {
+                        if let Some(last_left_col_index) = state.print_bytes((command.range.0, command.range.1)) {
                         state.index = last_left_col_index;
                         }
                         else {
@@ -1332,7 +807,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
                             continue;
                         };
 
-                        print_bytes(&state, state.range());
+                        state.print_bytes(state.range());
                     },
 
                     /* Quit */
@@ -1342,7 +817,7 @@ pub fn actual_runtime(filename:&str, quiet:bool, color:bool) -> i32 {
 
                     /* Print state */
                     's' => {
-                        print_state(&state);
+                        println!("{}", state);
                     },
 
                     /* Write out */
